@@ -1,14 +1,14 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 """ The GUI """
-import os, sys, Queue, threading, glob
+import os, sys, Queue, threading, glob, signal
 from contextlib import contextmanager
 from threading import RLock, Lock
 from urllib import unquote
 from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
 from PyQt5.Qt import (
-    QFileInfo, QObject, QBuffer, Qt, QByteArray, QTranslator,
-    QCoreApplication, QThread, QEvent, QTimer, pyqtSignal, QDateTime,
+    QFileInfo, QObject, QBuffer, Qt, QByteArray, QTranslator, QSocketNotifier,
+    QCoreApplication, QThread, QEvent, QTimer, pyqtSignal, QDateTime, QFontMetrics,
     QDesktopServices, QFileDialog, QFileIconProvider, QSettings, QIcon,
     QApplication, QDialog, QUrl, QFont, QFontDatabase, QLocale, QFontInfo)
 
@@ -16,7 +16,7 @@ ORG_NAME = 'KovidsBrain'
 APP_UID  = 'libprs500'
 from calibre import prints
 from calibre.constants import (islinux, iswindows, isbsd, isfrozen, isosx,
-        plugins, config_dir, filesystem_encoding, isxp)
+        plugins, config_dir, filesystem_encoding, isxp, DEBUG)
 from calibre.utils.config import Config, ConfigProxy, dynamic, JSONConfig
 from calibre.ebooks.metadata import MetaInformation
 from calibre.utils.date import UNDEFINED_DATE
@@ -140,12 +140,12 @@ defs['show_vl_tabs'] = False
 defs['show_highlight_toggle_button'] = False
 defs['add_comments_to_email'] = False
 defs['cb_preserve_aspect_ratio'] = False
-defs['show_rating_in_cover_browser'] = True
 defs['gpm_template_editor_font_size'] = 10
 defs['show_emblems'] = False
 defs['emblem_size'] = 32
 defs['emblem_position'] = 'left'
 defs['metadata_diff_mark_rejected'] = False
+defs['tag_browser_show_counts'] = True
 del defs
 # }}}
 
@@ -750,9 +750,11 @@ else:
                 ans = ans[0]
         return ans
 
-    def choose_images(window, name, title, select_only_single_file=True,
-                    formats=('png', 'gif', 'jpg', 'jpeg', 'svg')):
+    def choose_images(window, name, title, select_only_single_file=True, formats=None):
         mode = QFileDialog.ExistingFile if select_only_single_file else QFileDialog.ExistingFiles
+        if formats is None:
+            from calibre.gui2.dnd import image_extensions
+            formats = image_extensions()
         fd = FileDialog(title=title, name=name,
                         filters=[(_('Images'), list(formats))],
                         parent=window, add_all_files_filter=False, mode=mode,
@@ -860,6 +862,8 @@ def setup_gui_option_parser(parser):
 
 class Application(QApplication):
 
+    shutdown_signal_received = pyqtSignal()
+
     def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs):
         self.file_event_hook = None
         if override_program_name:
@@ -871,7 +875,27 @@ class Application(QApplication):
         self.headless = headless
         qargs = [i.encode('utf-8') if isinstance(i, unicode) else i for i in args]
         self.pi = plugins['progress_indicator'][0]
+        if not isosx and not headless and hasattr(Qt, 'AA_EnableHighDpiScaling'):
+            # On OS X high dpi scaling is turned on automatically by the OS, so we dont need to set it explicitly
+            # This requires Qt >= 5.6
+            for v in ('QT_AUTO_SCREEN_SCALE_FACTOR', 'QT_SCALE_FACTOR', 'QT_SCREEN_SCALE_FACTORS', 'QT_DEVICE_PIXEL_RATIO'):
+                if os.environ.get(v):
+                    break
+            else:
+                # Should probably make a preference to allow the user to
+                # control this, if needed.
+                # Could have options: auto, off, 1.25, 1.5, 1.75, 2, 2.25, 2.5
+                QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
         QApplication.__init__(self, qargs)
+        self.setAttribute(Qt.AA_UseHighDpiPixmaps)
+        if DEBUG and not headless:
+            prints('devicePixelRatio:', self.devicePixelRatio())
+            s = self.primaryScreen()
+            if s:
+                prints('logicalDpi:', s.logicalDotsPerInchX(), 'x', s.logicalDotsPerInchY())
+                prints('physicalDpi:', s.physicalDotsPerInchX(), 'x', s.physicalDotsPerInchY())
+        if not iswindows:
+            self.setup_unix_signals()
         if islinux or isbsd:
             self.setAttribute(Qt.AA_DontUseNativeMenuBar, 'CALIBRE_NO_NATIVE_MENUBAR' in os.environ)
         self.setup_styles(force_calibre_style)
@@ -891,6 +915,7 @@ class Application(QApplication):
             if s is not None:
                 font.setStretch(s)
             QApplication.setFont(font)
+        self.line_height = max(12, QFontMetrics(self.font()).lineSpacing())
 
         dl = QLocale(get_lang())
         if unicode(dl.bcp47Name()) != u'C':
@@ -1048,6 +1073,28 @@ class Application(QApplication):
     def __exit__(self, *args):
         self.setQuitOnLastWindowClosed(True)
 
+    def setup_unix_signals(self):
+        import fcntl
+        read_fd, write_fd = os.pipe()
+        cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
+        for fd in (read_fd, write_fd):
+            flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+            fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
+        for sig in (signal.SIGINT, signal.SIGTERM):
+            signal.signal(sig, lambda x, y: None)
+            signal.siginterrupt(sig, False)
+        signal.set_wakeup_fd(write_fd)
+        self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
+        self.signal_notifier.setEnabled(True)
+        self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+
+    def signal_received(self, read_fd):
+        try:
+            os.read(read_fd, 1024)
+        except EnvironmentError:
+            return
+        self.shutdown_signal_received.emit()
+
 _store_app = None
 
 @contextmanager
