@@ -9,8 +9,8 @@ from functools import partial
 
 from PyQt5.Qt import (QComboBox, QLabel, QSpinBox, QDoubleSpinBox, QDateTimeEdit,
         QDateTime, QGroupBox, QVBoxLayout, QSizePolicy, QGridLayout,
-        QSpacerItem, QIcon, QCheckBox, QWidget, QHBoxLayout,
-        QPushButton, QMessageBox, QToolButton, Qt)
+        QSpacerItem, QIcon, QCheckBox, QWidget, QHBoxLayout, QLineEdit,
+        QPushButton, QMessageBox, QToolButton, Qt, QPlainTextEdit)
 
 from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc
 from calibre.gui2.complete2 import EditWithComplete
@@ -21,6 +21,7 @@ from calibre.utils.config import tweaks
 from calibre.utils.icu import sort_key
 from calibre.library.comments import comments_to_html
 from calibre.gui2.library.delegates import ClearingDoubleSpinBox, ClearingSpinBox
+from calibre.gui2.widgets2 import RatingEditor
 
 class Base(object):
 
@@ -64,6 +65,37 @@ class Base(object):
     def break_cycles(self):
         self.db = self.widgets = self.initial_val = None
 
+class SimpleText(Base):
+
+    def setup_ui(self, parent):
+        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), QLineEdit(parent)]
+
+    def setter(self, val):
+        self.widgets[1].setText(type(u'')(val or ''))
+
+    def getter(self):
+        return self.widgets[1].text().strip()
+
+
+class LongText(Base):
+
+    def setup_ui(self, parent):
+        self._box = QGroupBox(parent)
+        self._box.setTitle('&'+self.col_metadata['name'])
+        self._layout = QVBoxLayout()
+        self._tb = QPlainTextEdit(self._box)
+        self._tb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
+        self._layout.addWidget(self._tb)
+        self._box.setLayout(self._layout)
+        self.widgets = [self._box]
+
+    def setter(self, val):
+        self._tb.setPlainText(type(u'')(val or ''))
+
+    def getter(self):
+        return self._tb.toPlainText()
+
+
 class Bool(Base):
 
     def setup_ui(self, parent):
@@ -127,27 +159,18 @@ class Float(Int):
             val = self.widgets[1].minimum()
         self.widgets[1].setValue(val)
 
-class Rating(Int):
+class Rating(Base):
 
     def setup_ui(self, parent):
-        Int.setup_ui(self, parent)
-        w = self.widgets[1]
-        w.setRange(0, 5)
-        w.setSuffix(' '+_('star(s)'))
-        w.setSpecialValueText(_('Unrated'))
+        allow_half_stars = self.col_metadata['display'].get('allow_half_stars', False)
+        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), RatingEditor(parent=parent, is_half_star=allow_half_stars)]
 
     def setter(self, val):
-        if val is None:
-            val = 0
-        self.widgets[1].setValue(int(round(val/2.)))
+        val = max(0, min(int(val or 0), 10))
+        self.widgets[1].rating_value = val
 
     def getter(self):
-        val = self.widgets[1].value()
-        if val == 0:
-            val = None
-        else:
-            val *= 2
-        return val
+        return self.widgets[1].rating_value or None
 
 class DateTimeEdit(QDateTimeEdit):
 
@@ -233,7 +256,7 @@ class Comments(Base):
         self._box = QGroupBox(parent)
         self._box.setTitle('&'+self.col_metadata['name'])
         self._layout = QVBoxLayout()
-        self._tb = CommentsEditor(self._box)
+        self._tb = CommentsEditor(self._box, toolbar_prefs_name=u'metadata-comments-editor-widget-hidden-toolbars')
         self._tb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
         # self._tb.setTabChangesFocus(True)
         self._layout.addWidget(self._tb)
@@ -349,7 +372,7 @@ class Text(Base):
         if self.col_metadata['is_multiple']:
             self.setter(val)
         else:
-            self.widgets[1].show_initial_value(val)
+            self.widgets[1].setText(val)
         self.initial_val = self.current_val
 
     def setter(self, val):
@@ -428,7 +451,7 @@ class Series(Base):
         val = self.normalize_db_val(val)
         self.name_widget.blockSignals(True)
         self.name_widget.update_items_cache(values)
-        self.name_widget.show_initial_value(val)
+        self.name_widget.setText(val)
         self.name_widget.blockSignals(False)
         self.initial_val, self.initial_index = self.current_val
 
@@ -511,6 +534,15 @@ class Enumeration(Base):
             val = None
         return val
 
+def comments_factory(db, key, parent):
+    fm = db.custom_column_num_map[key]
+    ctype = fm.get('display', {}).get('interpret_as', 'html')
+    if ctype == 'short-text':
+        return SimpleText(db, key, parent)
+    if ctype in ('long-text', 'markdown'):
+        return LongText(db, key, parent)
+    return Comments(db, key, parent)
+
 widgets = {
         'bool' : Bool,
         'rating' : Rating,
@@ -518,7 +550,7 @@ widgets = {
         'float': Float,
         'datetime': DateTime,
         'text' : Text,
-        'comments': Comments,
+        'comments': comments_factory,
         'series': Series,
         'enumeration': Enumeration
 }
@@ -526,7 +558,7 @@ widgets = {
 def field_sort_key(y, fm=None):
     m1 = fm[y]
     name = icu_lower(m1['name'])
-    n1 = 'zzzzz' + name if m1['datatype'] == 'comments' else name
+    n1 = 'zzzzz' + name if m1['datatype'] == 'comments' and m1.get('display', {}).get('interpret_as') != 'short-text' else name
     return sort_key(n1)
 
 def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, parent=None):
@@ -555,7 +587,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
         # Add the key if it really exists in the database
         if key in cols_to_display:
             cols.append(key)
-            if fm[key]['datatype'] == 'comments':
+            if fm[key]['datatype'] == 'comments' and fm[key].get('display', {}).get('interpret_as') != 'short-text':
                 comments_in_tweak += 1
 
     # Add all the remaining fields
@@ -563,7 +595,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
     for key in cols_to_display:
         if key not in cols:
             cols.append(key)
-            if fm[key]['datatype'] == 'comments':
+            if fm[key]['datatype'] == 'comments' and fm[key].get('display', {}).get('interpret_as') != 'short-text':
                 comments_not_in_tweak += 1
 
     count = len(cols)
@@ -582,9 +614,10 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
         dt = fm[key]['datatype']
         if dt == 'composite' or (bulk and dt == 'comments'):
             continue
+        is_comments = dt == 'comments' and fm[key].get('display', {}).get('interpret_as') != 'short-text'
         w = widget_factory(dt, fm[key]['colnum'])
         ans.append(w)
-        if two_column and dt == 'comments':
+        if two_column and is_comments:
             # Here for compatibility with old layout. Comments always started
             # in the left column
             comments_in_tweak -= 1
@@ -599,7 +632,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
                 comments_not_in_tweak = 0
 
         l = QGridLayout()
-        if dt == 'comments':
+        if is_comments:
             layout.addLayout(l, row, column, layout_rows_for_comments, 1)
             layout.setColumnStretch(column, 100)
             row += layout_rows_for_comments
@@ -608,7 +641,7 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
             layout.setColumnStretch(column, 100)
             row += 1
         for c in range(0, len(w.widgets), 2):
-            if dt != 'comments':
+            if not is_comments:
                 w.widgets[c].setWordWrap(True)
                 w.widgets[c].setBuddy(w.widgets[c+1])
                 l.addWidget(w.widgets[c], c, 0)
@@ -791,25 +824,16 @@ class BulkFloat(BulkInt):
 class BulkRating(BulkBase):
 
     def setup_ui(self, parent):
-        self.make_widgets(parent, QSpinBox)
-        self.main_widget.setRange(0, 5)
-        self.main_widget.setSuffix(' '+_('star(s)'))
-        self.main_widget.setSpecialValueText(_('Unrated'))
-        self.main_widget.setSingleStep(1)
+        allow_half_stars = self.col_metadata['display'].get('allow_half_stars', False)
+        self.make_widgets(parent, partial(RatingEditor, is_half_star=allow_half_stars))
 
     def setter(self, val):
-        if val is None:
-            val = 0
-        self.main_widget.setValue(int(round(val/2.)))
+        val = max(0, min(int(val or 0), 10))
+        self.main_widget.rating_value = val
         self.ignore_change_signals = False
 
     def getter(self):
-        val = self.main_widget.value()
-        if val == 0:
-            val = None
-        else:
-            val *= 2
-        return val
+        return self.main_widget.rating_value or None
 
 class BulkDateTime(BulkBase):
 
@@ -1048,7 +1072,7 @@ class BulkText(BulkBase):
             self.initial_val = val = self.normalize_db_val(val)
             self.ignore_change_signals = True
             self.main_widget.blockSignals(True)
-            self.main_widget.show_initial_value(val)
+            self.main_widget.setText(val)
             self.main_widget.blockSignals(False)
             self.ignore_change_signals = False
 
