@@ -26,12 +26,12 @@ from PyQt5.Qt import (
 from PyQt5.QtWebKitWidgets import QWebView
 
 from calibre.customize.ui import metadata_plugins
-from calibre.ebooks.metadata import authors_to_string
+from calibre.ebooks.metadata import authors_to_string, rating_to_stars
 from calibre.utils.logging import GUILog as Log
 from calibre.ebooks.metadata.sources.identify import urls_from_identifiers
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
-from calibre.gui2 import error_dialog, rating_font, gprefs, NO_URL_FORMATTING
+from calibre.gui2 import error_dialog, rating_font, gprefs
 from calibre.gui2.progress_indicator import draw_snake_spinner
 from calibre.utils.date import (utcnow, fromordinal, format_date,
         UNDEFINED_DATE, as_utc)
@@ -269,7 +269,7 @@ class ResultsView(QTableView):  # {{{
                 parts.append('<div>%s: %s</div>'%series)
         if not book.is_null('rating'):
             style = 'style=\'font-family:"%s"\''%f
-            parts.append('<div %s>%s</div>'%(style, '\u2605'*int(book.rating)))
+            parts.append('<div %s>%s</div>'%(style, rating_to_stars(int(book.rating))))
         parts.append('</center>')
         if book.identifiers:
             urls = urls_from_identifiers(book.identifiers)
@@ -326,7 +326,7 @@ class Comments(QWebView):  # {{{
 
     def link_clicked(self, url):
         from calibre.gui2 import open_url
-        if unicode(url.toString(NO_URL_FORMATTING)).startswith('http://'):
+        if url.scheme() in {'http', 'https'}:
             open_url(url)
 
     def turnoff_scrollbar(self, *args):
@@ -637,7 +637,7 @@ class CoversModel(QAbstractListModel):  # {{{
             current_cover = QPixmap(I('default_cover.png'))
         current_cover.setDevicePixelRatio(QApplication.instance().devicePixelRatio())
 
-        self.blank = QPixmap(I('blank.png')).scaled(*CoverDelegate.ICON_SIZE)
+        self.blank = QIcon(I('blank.png')).pixmap(*CoverDelegate.ICON_SIZE)
         self.cc = current_cover
         self.reset_covers(do_reset=False)
 
