@@ -16,7 +16,7 @@ from calibre.gui2 import (
     Application, ORG_NAME, APP_UID, choose_files, info_dialog, error_dialog,
     open_url, setup_gui_option_parser)
 from calibre.ebooks.oeb.iterator.book import EbookIterator
-from calibre.constants import islinux, filesystem_encoding, DEBUG
+from calibre.constants import islinux, filesystem_encoding, DEBUG, iswindows
 from calibre.utils.config import Config, StringConfig, JSONConfig
 from calibre.customize.ui import available_input_formats
 from calibre import as_unicode, force_unicode, isbytestring, prints
@@ -178,6 +178,7 @@ class EbookViewer(MainWindow):
         self.action_reload = QAction(_('&Reload book'), self)
         self.action_reload.triggered.connect(self.reload_book)
         self.action_quit.triggered.connect(self.quit)
+        QApplication.instance().shutdown_signal_received.connect(self.action_quit.trigger)
         self.action_reference_mode.triggered[bool].connect(self.view.reference_mode)
         self.action_metadata.triggered[bool].connect(self.metadata.setVisible)
         self.action_table_of_contents.toggled[bool].connect(self.set_toc_visible)
@@ -953,10 +954,9 @@ class EbookViewer(MainWindow):
                         det_msg=tb, show=True)
             self.close_progress_indicator()
         else:
-            self.metadata.show_opf(self.iterator.opf,
-                    self.iterator.book_format)
+            self.metadata.show_metadata(self.iterator.mi, self.iterator.book_format)
             self.view.current_language = self.iterator.language
-            title = self.iterator.opf.title
+            title = self.iterator.mi.title
             if not title:
                 title = os.path.splitext(os.path.basename(pathtoebook))[0]
             if self.iterator.toc:
@@ -1207,6 +1207,15 @@ def main(args=sys.argv):
     # Ensure viewer can continue to function if GUI is closed
     os.environ.pop('CALIBRE_WORKER_TEMP_DIR', None)
     reset_base_dir()
+    if iswindows:
+        # Ensure that all ebook editor instances are grouped together in the task
+        # bar. This prevents them from being grouped with viewer process when
+        # launched from within calibre, as both use calibre-parallel.exe
+        import ctypes
+        try:
+            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID('com.calibre-ebook.viewer')
+        except Exception:
+            pass  # Only available on windows 7 and newer
 
     parser = option_parser()
     opts, args = parser.parse_args(args)
